"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onFirestoreCreate = onFirestoreCreate;
const internal_methods_1 = require("../internal-methods");
const types_1 = require("../types");
/**
 * Decorator that adds the method to the list of Cloud Functions triggered when creating new
 * document in Firestore.
 *
 * @param document Firestore document/collection path or complete options.
 *                 To use wildcard keys, enter the parameters between keys.
 *                 e.g. 'user/{uid}/account/{accountId}'
 * @example
 * import { FirestoreEvent, QueryDocumentSnapshot } from 'firebase-functions/v2/firestore';
 * import { ParamsOf } from 'firebase-functions/lib/common/params';
 * import { onFirestoreCreate } from 'firebase-triggers';
 *
 * class TodoCtrl {
 *     \@onFirestoreCreate('todo/{id}')
 *     onCreate(event: FirestoreEvent<QueryDocumentSnapshot, ParamsOf<string>>) {
 *         // Get an object representing the document. e.g. { title: 'Wash the dishes', time: '12:00' }
 *         const newValue = event.data.data();
 *         // access a particular field as you would any JS property
 *         const title = newValue.title;
 *         const time = newValue.time;
 *
 *         console.log(`New task added: ${title} at ${time}`);
 *     }
 * }
 */
function onFirestoreCreate(document) {
    return (target, key) => {
        const options = typeof document === 'string' ? { document } : document;
        const firebaseFunction = {
            className: (0, internal_methods_1.getClassName)(target),
            methodName: key,
            method: (0, internal_methods_1.getClassMethod)(target, key),
            trigger: types_1.FirebaseTriggerType.FIRESTORE_CREATE,
            key: options.document,
            options,
        };
        (0, internal_methods_1.addFirebaseFunction)(firebaseFunction);
        Reflect.defineMetadata('onFirestoreCreate', options, target, key);
    };
}
//# sourceMappingURL=firestore-create.js.map