"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onFirestoreDelete = onFirestoreDelete;
const internal_methods_1 = require("../internal-methods");
const types_1 = require("../types");
/**
 * Decorator that adds the method to the list of Cloud Functions triggered when removing a document
 * from the Firestore.
 *
 * @param document Firestore document/collection path or complete options.
 *                 To use wildcard keys, enter the parameters between keys.
 *                 e.g. 'user/{uid}/account/{accountId}'
 * @example
 * import { FirestoreEvent, QueryDocumentSnapshot } from 'firebase-functions/v2/firestore';
 * import { ParamsOf } from 'firebase-functions/lib/common/params';
 * import { onFirestoreDelete } from 'firebase-triggers';
 *
 * class TodoCtrl {
 *     \@onFirestoreDelete('todo/{id}')
 *     onDelete(event: FirestoreEvent<QueryDocumentSnapshot, ParamsOf<string>>) {
 *         // Get an object representing the document. e.g. { title: 'Wash the dishes', time: '12:00' }
 *         const oldValue = event.data.data();
 *         // access a particular field as you would any JS property
 *         const title = oldValue.title;
 *
 *         console.log(`Task "${title}" removed`);
 *     }
 * }
 */
function onFirestoreDelete(document) {
    const options = typeof document === 'string' ? { document } : document;
    return (target, key) => {
        const firebaseFunction = {
            className: (0, internal_methods_1.getClassName)(target),
            methodName: key,
            method: (0, internal_methods_1.getClassMethod)(target, key),
            trigger: types_1.FirebaseTriggerType.FIRESTORE_DELETE,
            key: options.document,
            options,
        };
        (0, internal_methods_1.addFirebaseFunction)(firebaseFunction);
        Reflect.defineMetadata('onFirestoreDelete', options, target, key);
    };
}
//# sourceMappingURL=firestore-delete.js.map