"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onFirestoreUpdate = onFirestoreUpdate;
const internal_methods_1 = require("../internal-methods");
const types_1 = require("../types");
/**
 * Decorator that adds the method to the Cloud Functions list triggered when editing an existing
 * document in Firestore.
 *
 * @param document Firestore document/collection path os complete options.
 *                 To use wildcard keys, enter the parameters between keys.
 *                 e.g. 'user/{uid}/account/{accountId}'
 * @example
 * import { FirestoreEvent, QueryDocumentSnapshot } from 'firebase-functions/v2/firestore';
 * import { Change } from 'firebase-functions';
 * import { ParamsOf } from 'firebase-functions/lib/common/params';
 * import { onFirestoreUpdate } from 'firebase-triggers';
 *
 * class TodoCtrl {
 *     \@onFirestoreUpdate('todo/{id}')
 *     onUpdate(event: FirestoreEvent<Change<QueryDocumentSnapshot>, ParamsOf<string>>) {
 *         // Get an object representing the document. e.g. { title: 'Wash the dishes', time: '12:00' }
 *         const newValue = event.data.after.data();
 *         // ...or the previous value before this update
 *         const previousValue = event.data.before.data();
 *         // access a particular field as you would any JS property
 *         const newTitle = newValue.title;
 *         const oldTitle = previousValue.title;
 *
 *         console.log(`Changed the title from "${oldTitle}" to "${newTitle}"`);
 *     }
 * }
 */
function onFirestoreUpdate(document) {
    const options = typeof document === 'string' ? { document } : document;
    return (target, key) => {
        const firebaseFunction = {
            className: (0, internal_methods_1.getClassName)(target),
            methodName: key,
            method: (0, internal_methods_1.getClassMethod)(target, key),
            trigger: types_1.FirebaseTriggerType.FIRESTORE_UPDATE,
            key: options.document,
            options,
        };
        (0, internal_methods_1.addFirebaseFunction)(firebaseFunction);
        Reflect.defineMetadata('onFirestoreUpdate', options, target, key);
    };
}
//# sourceMappingURL=firestore-update.js.map