"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onFirestoreWrite = onFirestoreWrite;
const internal_methods_1 = require("../internal-methods");
const types_1 = require("../types");
/**
 * Decorator that adds the method to the list of Cloud Functions triggered when creating, editing
 * or removing documents in Firestore.
 *
 * @param document Firestore document or collection path
 *                 To use wildcard keys, enter the parameters between keys.
 *                 e.g. 'user/{uid}/account/{accountId}'
 * @example
 * import { FirestoreEvent, QueryDocumentSnapshot } from 'firebase-functions/v2/firestore';
 * import { Change } from 'firebase-functions';
 * import { ParamsOf } from 'firebase-functions/lib/common/params';
 * import { onFirestoreWrite } from 'firebase-triggers';
 *
 * class TodoCtrl {
 *     \@onFirestoreWrite('todo/{id}')
 *     onWrite(event: FirestoreEvent<Change<DocumentSnapshot>, ParamsOf<string>>) {
 *         // Get an object with the current document value. If the document does not exist, it has been deleted.
 *         const newDocument = event.data.after.exists ? event.data.after.data() : null;
 *         // Get an object with the previous document value (for update or delete)
 *         const oldDocument = event.data.before.exists ? event.data.before.data() : null;
 *
 *         if (!newDocument) {
 *             const title = oldDocument.title;
 *             console.log(`Task "${title}" removed`);
 *             return;
 *         }
 *
 *         if (!oldDocument) {
 *             const title = newDocument.title;
 *             const time = newDocument.time;
 *             console.log(`New task added: ${title} at ${time}`);
 *             return;
 *         }
 *
 *         const newTitle = newDocument.title;
 *         const oldTitle = oldDocument.title;
 *
 *         console.log(`Changed the title from "${oldTitle}" to "${newTitle}"`);
 *     }
 * }
 */
function onFirestoreWrite(document) {
    const options = typeof document === 'string' ? { document } : document;
    return (target, key) => {
        const firebaseFunction = {
            className: (0, internal_methods_1.getClassName)(target),
            methodName: key,
            method: (0, internal_methods_1.getClassMethod)(target, key),
            trigger: types_1.FirebaseTriggerType.FIRESTORE_WRITE,
            key: options.document,
            options,
        };
        (0, internal_methods_1.addFirebaseFunction)(firebaseFunction);
        Reflect.defineMetadata('onFirestoreWrite', options, target, key);
    };
}
//# sourceMappingURL=firestore-write.js.map