"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onPubSubPublish = onPubSubPublish;
const internal_methods_1 = require("../internal-methods");
const types_1 = require("../types");
/**
 * Decorator that adds the method to the list of Cloud Functions triggered when a publication is
 * made via PubSub on the specified topic.
 *
 * @param topic PubSub topic to subscribe
 * @example
 * import { CloudEvent } from 'firebase-functions/lib/v2/core';
 * import { MessagePublishedData } from 'firebase-functions/v2/pubsub';
 * import { onPubSubPublish } from 'firebase-triggers';
 *
 * class SampleCtrl {
 *     \@onPubSubPublish('my-topic')
 *     doSomething(event: CloudEvent<MessagePublishedData<any>>) {
 *         const publishedData = message.json;
 *         console.log('Data published via PubSub on my-topic:', publishedData);
 *     }
 * }
 */
function onPubSubPublish(topic) {
    const options = typeof topic === 'string' ? { topic } : topic;
    return (target, key) => {
        const firebaseFunction = {
            className: (0, internal_methods_1.getClassName)(target),
            methodName: key,
            method: (0, internal_methods_1.getClassMethod)(target, key),
            trigger: types_1.FirebaseTriggerType.PUBSUB_PUBLISH,
            key: options.topic,
            options,
        };
        (0, internal_methods_1.addFirebaseFunction)(firebaseFunction);
        Reflect.defineMetadata('onPubSubPublish', options, target, key);
    };
}
//# sourceMappingURL=pubsub-publish.js.map