"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onPubSubSchedule = onPubSubSchedule;
const internal_methods_1 = require("../internal-methods");
const types_1 = require("../types");
/**
 * Decorator that adds the method to the list of Cloud Functions triggered when a publication is
 * made via PubSub triggered via Cron.
 *
 * @param schedule Cron time interval (ex: '5 11 * * *' ou 'every 5 minutes')
 * @example
 * import { ScheduledEvent } from 'firebase-functions/v2/scheduler';
 * import { onPubSubSchedule } from 'firebase-triggers';
 *
 * class TimerCtrl {
 *     \@onPubSubSchedule('0 5 * * *')
 *     everyDayAtFiveAM(event: ScheduledEvent) {
 *         console.log('Method executed every day at 5 AM');
 *     }
 * }
 */
function onPubSubSchedule(schedule) {
    const options = typeof schedule === 'string' ? { schedule } : schedule;
    return (target, key) => {
        const firebaseFunction = {
            className: (0, internal_methods_1.getClassName)(target),
            methodName: key,
            method: (0, internal_methods_1.getClassMethod)(target, key),
            trigger: types_1.FirebaseTriggerType.PUBSUB_SCHEDULE,
            key: options.schedule,
            options,
        };
        (0, internal_methods_1.addFirebaseFunction)(firebaseFunction);
        Reflect.defineMetadata('onPubSubSchedule', options, target, key);
    };
}
//# sourceMappingURL=pubsub-schedule.js.map