import { HttpsOptions } from 'firebase-functions/v2/https';
export type HTTPMethod = 'GET' | 'POST' | 'PUT' | 'PATCH' | 'DELETE' | string;
export type RequestOptions = {
    path?: string;
    methods?: HTTPMethod | HTTPMethod[];
};
/**
 * Decorator that adds the method to the Cloud Functions list triggered when receiving an HTTP
 * request.
 *
 * @param opt Optional function settings. If you enter a string it is used as a path.
 * @param opt.methods HTTP methods accepted in the request.
 * @param opt.path URL suffix also used as method name in GCP. (optional)
 *                 If you do not enter the path, the method name will be used. (Recommended)
 * @example
 * import { Response } from 'firebase-functions';
 * import { onRequest } from 'firebase-triggers';
 * import { Request } from 'firebase-functions/lib/common/providers/https';
 *
 * class UserCtrl {
 *     \@onRequest()
 *     async profile(request: Request, response: Response) {
 *         const profile = await loadProfile(request.body.id);
 *         response.json(profile);
 *     }
 * }
 */
export declare function onRequest(opt?: string | RequestOptions & HttpsOptions): (target: any, key: string) => void;
/**
 * Decorator that adds the method to the Cloud Functions list triggered when receiving an GET HTTP
 * request.
 *
 * @param path URL suffix also used as method name in GCP. (optional)
 *             If you do not enter the path, the method name will be used. (Recommended)
 *
 * @example
 * import { Request } from 'firebase-functions/lib/common/providers/https';
 * import { Response } from 'firebase-functions';
 * import { GET } from 'firebase-triggers';
 *
 * class UserCtrl {
 *     \@GET('users')
 *     async get(request: Request, response: Response) {
 *         response.json([]);
 *     }
 * }
 */
export declare function GET(path?: string | HttpsOptions & {
    path?: string;
}): (target: any, key: string) => void;
/**
 * Decorator that adds the method to the Cloud Functions list triggered when receiving an POST HTTP
 * request.
 *
 * @param path URL suffix also used as method name in GCP. (optional)
 *             If you do not enter the path, the method name will be used. (Recommended)
 *
 * @example
 * import { Request } from 'firebase-functions/lib/common/providers/https';
 * import { Response } from 'firebase-functions';
 * import { POST } from 'firebase-triggers';
 *
 * class UserCtrl {
 *     \@POST('users')
 *     async post(request: Request, response: Response) {
 *         response.status(201).send();
 *     }
 * }
 */
export declare function POST(path?: string | HttpsOptions & {
    path?: string;
}): (target: any, key: string) => void;
/**
 * Decorator that adds the method to the Cloud Functions list triggered when receiving an PUT HTTP
 * request.
 *
 * @param path URL suffix also used as method name in GCP. (optional)
 *             If you do not enter the path, the method name will be used. (Recommended)
 *
 * @example
 * import { Request } from 'firebase-functions/lib/common/providers/https';
 * import { Response } from 'firebase-functions';
 * import { PUT } from 'firebase-triggers';
 *
 * class UserCtrl {
 *     \@PUT('users')
 *     async put(request: Request, response: Response) {
 *         response.status(201).send();
 *     }
 * }
 */
export declare function PUT(path?: string | HttpsOptions & {
    path?: string;
}): (target: any, key: string) => void;
/**
 * Decorator that adds the method to the Cloud Functions list triggered when receiving an PATCH HTTP request
 * @param path URL suffix also used as method name in GCP. (optional)
 *             If you do not enter the path, the method name will be used. (Recommended)
 *
 * @example
 * import { Request } from 'firebase-functions/lib/common/providers/https';
 * import { Response } from 'firebase-functions';
 * import { PATCH } from 'firebase-triggers';
 *
 * class UserCtrl {
 *     \@PATCH('users')
 *     async patch(request: Request, response: Response) {
 *         response.status(201).send();
 *     }
 * }
 */
export declare function PATCH(path?: string | HttpsOptions & {
    path?: string;
}): (target: any, key: string) => void;
/**
 * Decorator that adds the method to the Cloud Functions list triggered when receiving an DELETE HTTP request
 * @param path URL suffix also used as method name in GCP. (optional)
 *             If you do not enter the path, the method name will be used. (Recommended)
 *
 * @example
 * import { Request } from 'firebase-functions/lib/common/providers/https';
 * import { Response } from 'firebase-functions';
 * import { DELETE } from 'firebase-triggers';
 *
 * class UserCtrl {
 *     @DELETE('users')
 *     async del(request: Request, response: Response) {
 *         response.status(201).send();
 *     }
 * }
 */
export declare function DELETE(path?: string | HttpsOptions & {
    path?: string;
}): (target: any, key: string) => void;
