"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onRequest = onRequest;
exports.GET = GET;
exports.POST = POST;
exports.PUT = PUT;
exports.PATCH = PATCH;
exports.DELETE = DELETE;
const internal_methods_1 = require("../internal-methods");
const types_1 = require("../types");
/**
 * Returns a FirebaseFunction based on a specific method.
 *
 * @param httpMethod HTTP method
 * @param target
 * @param key
 * @param path URL suffix also used as method name in GCP. (optional)
 *             If you do not enter the path, the method name will be used. (Recommended)
 */
function getSpecificMethod(httpMethod, target, key, opt = {}) {
    const path = opt.path;
    const options = { ...opt };
    delete options.path;
    return {
        className: (0, internal_methods_1.getClassName)(target),
        methodName: key,
        method: (0, internal_methods_1.getClassMethod)(target, key),
        trigger: types_1.FirebaseTriggerType.HTTP_REQUEST,
        key: { methods: httpMethod, ...(!!path && { path }) },
        options,
    };
}
function extractRequestOptions(options = {}) {
    if (options && typeof options === 'object') {
        const { path, methods } = options;
        return {
            ...(path && { path }),
            ...(methods && { methods }),
        };
    }
    return { path: options };
}
function extractHttpsOptions(options) {
    if (!options || typeof options === 'string') {
        return {};
    }
    const result = { ...options };
    delete result.path;
    delete result.methods;
    return result;
}
/**
 * Decorator that adds the method to the Cloud Functions list triggered when receiving an HTTP
 * request.
 *
 * @param opt Optional function settings. If you enter a string it is used as a path.
 * @param opt.methods HTTP methods accepted in the request.
 * @param opt.path URL suffix also used as method name in GCP. (optional)
 *                 If you do not enter the path, the method name will be used. (Recommended)
 * @example
 * import { Response } from 'firebase-functions';
 * import { onRequest } from 'firebase-triggers';
 * import { Request } from 'firebase-functions/lib/common/providers/https';
 *
 * class UserCtrl {
 *     \@onRequest()
 *     async profile(request: Request, response: Response) {
 *         const profile = await loadProfile(request.body.id);
 *         response.json(profile);
 *     }
 * }
 */
function onRequest(opt = {}) {
    const requestOptions = extractRequestOptions(opt);
    const options = extractHttpsOptions(opt);
    return (target, key) => {
        const firebaseFunction = {
            className: (0, internal_methods_1.getClassName)(target),
            methodName: key,
            method: (0, internal_methods_1.getClassMethod)(target, key),
            trigger: types_1.FirebaseTriggerType.HTTP_REQUEST,
            key: requestOptions,
            options,
        };
        (0, internal_methods_1.addFirebaseFunction)(firebaseFunction);
        Reflect.defineMetadata('onRequest', opt, target, key);
    };
}
/**
 * Decorator that adds the method to the Cloud Functions list triggered when receiving an GET HTTP
 * request.
 *
 * @param path URL suffix also used as method name in GCP. (optional)
 *             If you do not enter the path, the method name will be used. (Recommended)
 *
 * @example
 * import { Request } from 'firebase-functions/lib/common/providers/https';
 * import { Response } from 'firebase-functions';
 * import { GET } from 'firebase-triggers';
 *
 * class UserCtrl {
 *     \@GET('users')
 *     async get(request: Request, response: Response) {
 *         response.json([]);
 *     }
 * }
 */
function GET(path = {}) {
    return (target, key) => {
        const options = typeof path === 'string' ? { path } : path;
        (0, internal_methods_1.addFirebaseFunction)(getSpecificMethod('GET', target, key, options));
        Reflect.defineMetadata('GET', options, target, key);
    };
}
/**
 * Decorator that adds the method to the Cloud Functions list triggered when receiving an POST HTTP
 * request.
 *
 * @param path URL suffix also used as method name in GCP. (optional)
 *             If you do not enter the path, the method name will be used. (Recommended)
 *
 * @example
 * import { Request } from 'firebase-functions/lib/common/providers/https';
 * import { Response } from 'firebase-functions';
 * import { POST } from 'firebase-triggers';
 *
 * class UserCtrl {
 *     \@POST('users')
 *     async post(request: Request, response: Response) {
 *         response.status(201).send();
 *     }
 * }
 */
function POST(path = {}) {
    return (target, key) => {
        const options = typeof path === 'string' ? { path } : path;
        (0, internal_methods_1.addFirebaseFunction)(getSpecificMethod('POST', target, key, options));
        Reflect.defineMetadata('POST', options, target, key);
    };
}
/**
 * Decorator that adds the method to the Cloud Functions list triggered when receiving an PUT HTTP
 * request.
 *
 * @param path URL suffix also used as method name in GCP. (optional)
 *             If you do not enter the path, the method name will be used. (Recommended)
 *
 * @example
 * import { Request } from 'firebase-functions/lib/common/providers/https';
 * import { Response } from 'firebase-functions';
 * import { PUT } from 'firebase-triggers';
 *
 * class UserCtrl {
 *     \@PUT('users')
 *     async put(request: Request, response: Response) {
 *         response.status(201).send();
 *     }
 * }
 */
function PUT(path = {}) {
    return (target, key) => {
        const options = typeof path === 'string' ? { path } : path;
        (0, internal_methods_1.addFirebaseFunction)(getSpecificMethod('PUT', target, key, options));
        Reflect.defineMetadata('PUT', options, target, key);
    };
}
/**
 * Decorator that adds the method to the Cloud Functions list triggered when receiving an PATCH HTTP request
 * @param path URL suffix also used as method name in GCP. (optional)
 *             If you do not enter the path, the method name will be used. (Recommended)
 *
 * @example
 * import { Request } from 'firebase-functions/lib/common/providers/https';
 * import { Response } from 'firebase-functions';
 * import { PATCH } from 'firebase-triggers';
 *
 * class UserCtrl {
 *     \@PATCH('users')
 *     async patch(request: Request, response: Response) {
 *         response.status(201).send();
 *     }
 * }
 */
function PATCH(path = {}) {
    return (target, key) => {
        const options = typeof path === 'string' ? { path } : path;
        (0, internal_methods_1.addFirebaseFunction)(getSpecificMethod('PATCH', target, key, options));
        Reflect.defineMetadata('PATCH', options, target, key);
    };
}
/**
 * Decorator that adds the method to the Cloud Functions list triggered when receiving an DELETE HTTP request
 * @param path URL suffix also used as method name in GCP. (optional)
 *             If you do not enter the path, the method name will be used. (Recommended)
 *
 * @example
 * import { Request } from 'firebase-functions/lib/common/providers/https';
 * import { Response } from 'firebase-functions';
 * import { DELETE } from 'firebase-triggers';
 *
 * class UserCtrl {
 *     @DELETE('users')
 *     async del(request: Request, response: Response) {
 *         response.status(201).send();
 *     }
 * }
 */
function DELETE(path = {}) {
    return (target, key) => {
        const options = typeof path === 'string' ? { path } : path;
        (0, internal_methods_1.addFirebaseFunction)(getSpecificMethod('DELETE', target, key, options));
        Reflect.defineMetadata('DELETE', options, target, key);
    };
}
//# sourceMappingURL=request.js.map