"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFirebaseFunctionListToExport = getFirebaseFunctionListToExport;
const functionsV1 = require("firebase-functions/v1");
const scheduler_1 = require("firebase-functions/v2/scheduler");
const types_1 = require("./types");
const https_1 = require("firebase-functions/v2/https");
const firestore_1 = require("firebase-functions/v2/firestore");
const pubsub_1 = require("firebase-functions/v2/pubsub");
const storage_1 = require("firebase-functions/v2/storage");
const fs_1 = require("fs");
const path_1 = require("path");
const handlers_1 = require("./handlers");
const multiple_request_handler_1 = require("./handlers/multiple-request-handler");
const request_schema_validator_handler_1 = require("./handlers/request-schema-validator-handler");
const internal_methods_1 = require("./internal-methods");
const identity_1 = require("firebase-functions/identity");
/**
 * Get Firebase Function Builder setting the Firebase Runtime Options.
 *
 * @param options Firebase runtime options
 * @returns Firebase Function Builder
 */
function getFunctionBuilder(options = {}) {
    const parsedOptions = { ...options };
    delete parsedOptions.region;
    let result = functionsV1.runWith(parsedOptions);
    if (options.region) {
        const regions = typeof options.region === 'string' ? [options.region] : options.region;
        if (regions.length) {
            result = result.region(...regions);
        }
    }
    return result;
}
/** Methods used to register Firebase triggers */
const triggerMethods = {
    CALLABLE: (handler, options = {}) => (0, https_1.onCall)(options, handler),
    USER_CREATE: (handler, options = {}) => (0, identity_1.beforeUserCreated)(options, handler),
    USER_DELETE: (handler, options) => getFunctionBuilder(options).auth.user().onDelete(handler),
    HTTP_REQUEST: (handler, options = {}) => (0, https_1.onRequest)(options, handler),
    FIRESTORE_CREATE: (handler, options) => (0, firestore_1.onDocumentCreated)(options, handler),
    FIRESTORE_UPDATE: (handler, options) => (0, firestore_1.onDocumentUpdated)(options, handler),
    FIRESTORE_DELETE: (handler, options) => (0, firestore_1.onDocumentDeleted)(options, handler),
    FIRESTORE_WRITE: (handler, options) => (0, firestore_1.onDocumentWritten)(options, handler),
    PUBSUB_PUBLISH: (handler, options) => (0, pubsub_1.onMessagePublished)(options, handler),
    PUBSUB_SCHEDULE: (handler, options) => (0, scheduler_1.onSchedule)(options, handler),
    STORAGE_ARCHIVE: (handler, options) => (0, storage_1.onObjectArchived)(options, handler),
    STORAGE_DELETE: (handler, options) => (0, storage_1.onObjectDeleted)(options, handler),
    STORAGE_FINALIZE: (handler, options) => (0, storage_1.onObjectFinalized)(options, handler),
    STORAGE_METADATA_UPDATE: (handler, options) => (0, storage_1.onObjectMetadataUpdated)(options, handler),
};
/**
 * Returns function name for Cloud Functions based on the method name.
 * If the function's trigger is for HTTP requests and the parameter with the method name is passed,
 * then this nomenclature rule will be ignored and the name of the method entered will be used.
 * @param func Function data
 */
function getCloudFunctionName(func) {
    if (func.trigger === types_1.FirebaseTriggerType.HTTP_REQUEST && func.key) {
        const path = typeof func.key === 'string' ? func.key : func.key.path;
        if (path) {
            return path;
        }
    }
    return func.methodName;
}
function getHTTPMethodHandler(fullMethodName, httpRequestFunctions) {
    const requestHandlerList = {};
    Object.keys(httpRequestFunctions).forEach((httpMethod) => {
        let specificMethod = httpRequestFunctions[httpMethod].handler;
        const schemaFileFileName = `${fullMethodName}${httpMethod === 'DEFAULT' ? '' : `_${httpMethod}`}.json`;
        const schemaFile = (0, path_1.resolve)(`schema/${schemaFileFileName}`);
        if ((0, fs_1.existsSync)(schemaFile)) {
            specificMethod = (0, request_schema_validator_handler_1.requestSchemaValidatorHandler)(specificMethod, schemaFile);
        }
        requestHandlerList[httpMethod] = specificMethod;
    });
    let method = (0, multiple_request_handler_1.multipleRequestHandler)(requestHandlerList);
    method = (0, handlers_1.requestErrorHandler)(method);
    return method;
}
/**
 * Returns a list of methods linked to triggers to export at application startup.
 *
 * @example
 * import 'reflect-metadata';
 * import { getFirebaseFunctionListToExport } from 'firebase-triggers';
 *
 * const list = getFirebaseFunctionListToExport();
 * for (const key in list) {
 *   exports[key] = list[key];
 * }
 */
function getFirebaseFunctionListToExport() {
    const result = {};
    const httpRequestFunctions = {};
    const functionList = (0, internal_methods_1.getFirebaseFunctionList)();
    // Add methods to export, except HTTP methods
    functionList.forEach((func) => {
        const triggerMethod = triggerMethods[func.trigger];
        if (triggerMethod) {
            const groupName = (0, internal_methods_1.getGroupName)(func);
            const cloudFunctionName = getCloudFunctionName(func);
            const fullMethodName = groupName ? `${groupName}-${cloudFunctionName}` : cloudFunctionName;
            if (func.trigger === types_1.FirebaseTriggerType.HTTP_REQUEST) {
                const methods = !func.key
                    ? ['DEFAULT']
                    : Array.isArray(func.key.methods)
                        ? func.key.methods
                        : [func.key.methods || 'DEFAULT'];
                if (!httpRequestFunctions[fullMethodName]) {
                    httpRequestFunctions[fullMethodName] = {};
                }
                methods.forEach((httpMethod) => {
                    httpRequestFunctions[fullMethodName][httpMethod] = {
                        handler: func.method,
                        options: func.options,
                    };
                });
                return;
            }
            if (groupName) {
                if (!result[groupName]) {
                    result[groupName] = {};
                }
                result[groupName][cloudFunctionName] = triggerMethod(func.method, func.options);
            }
            else {
                result[cloudFunctionName] = triggerMethod(func.method, func.options);
            }
        }
    });
    // Add HTTP methods to export
    Object.keys(httpRequestFunctions).forEach((fullMethodName) => {
        const name = fullMethodName.split('-');
        const groupName = name.length > 1 ? name[0] : undefined;
        const cloudFunctionName = name.length > 1 ? name[1] : name[0];
        const methodHandler = getHTTPMethodHandler(fullMethodName, httpRequestFunctions[fullMethodName]);
        const options = Object
            .values(httpRequestFunctions[fullMethodName])
            .find(item => !!item.options) || {};
        if (groupName) {
            if (!result[groupName]) {
                result[groupName] = {};
            }
            result[groupName][cloudFunctionName] = (0, https_1.onRequest)(options, methodHandler);
        }
        else {
            result[cloudFunctionName] = (0, https_1.onRequest)(options, methodHandler);
        }
    });
    return result;
}
//# sourceMappingURL=functions.js.map